/*
Ajouts LCD / F1FWG

LCD : http://www.arduino.cc/en/Tutorial/LiquidCrystal

BMP085 : Get pressure, altitude, and temperature from the BMP085.
 Serial.print it out at 9600 baud to serial monitor.
 Fixed for Arduino 1.0+ by iLabBali.com
 Based largely on code by Jim Lindblom via the repost at
 http://bildr.org/2011/06/bmp085-arduino/
*/

//**********************************************************************************
// BIBLIOTHEQUES
//
#include <Streaming_ger.h>
// bus I2C
#include <Wire.h>
// LCD
#include <LiquidCrystal.h>

// BMP085
#define BMP085_ADDRESS 0x77  // I2C address of BMP085
const unsigned char OSS = 2;  // Oversampling Setting
// les ports de l'I2C sont A4(SCL) et A5(SDA)

// Calibration values
int ac1;
int ac2;
int ac3;
unsigned int ac4;
unsigned int ac5;
unsigned int ac6;
int b1;
int b2;
int mb;
int mc;
int md;
// b5 is calculated in bmp085GetTemperature(...), this variable is also used in bmp085GetPressure(...)
// so ...Temperature(...) must be called before ...Pressure(...).
long b5; 

  float temperature,pressure,atm,altitude;
  char valuef[15] = "99.99\0";  

  unsigned long millissuiv;

// LCD initialize the library with the numbers of the interface pins
/*
 * LCD pin 1 to GND
 * LCD pin 2 to VCC 5V
 * LCD pin 3 : contraste sur le point milieu d'un potentiomètre de 10K entre le GND et le VCC (réglé très près de la masse)
 * LCD pin 4 RS to digital pin 4
 * LCD pin 5 RW to GND
 * LCD pin 6 Enable pin to digital pin 2
 * LCD pin 11 D4 pin to digital pin 6
 * LCD pin 12 D5 pin to digital pin 7
 * LCD pin 13 D6 pin to digital pin 8
 * LCD pin 14 D7 pin to digital pin 9
*/
// (RS, Enable,  D4, D5, D6, D7)
  LiquidCrystal lcd(4, 2, 6, 7, 8, 9);

//**********************************************************************************
// INITIALISATIONS
void setup(){
// voie serie
  Serial.begin(9600);
// set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);  lcd && "LCD";
// I2C
  Wire.begin();
  lcd.setCursor(4, 0);  lcd && "I2C";
// BMP085
  bmp085Calibration();
  lcd.setCursor(8, 0);  lcd && "BMP085";
}

//**********************************************************************************
// PROGRAMME PRINCIPAL
void loop()
{
if (millis() > millissuiv) {
  millissuiv = millis()+1000;
  
  temperature = bmp085GetTemperature(bmp085ReadUT()); //MUST be called first
  pressure = bmp085GetPressure(bmp085ReadUP());
  atm = pressure / 101325; // "standard atmosphere"
  altitude = calcAltitude(pressure); //Uncompensated caculation - in Meters 

  Serial && _FLOAT(temperature,1) && "^C " && _FLOAT(pressure/100,0) && "hPa " && _FLOAT(atm,3) && "Atm " && _FLOAT(altitude,0) && "m" && endli ;
// 'xF8' degrés ??

  lcd.setCursor(0, 0); 
  lcd && _FLOAT(temperature,1) && "^C  " && _FLOAT(pressure/100,0) && "hPa ";
  lcd.setCursor(0, 1); 
  lcd && _FLOAT(millis()/1000,0) && "   ";
  
/*
  Serial.print(temperature, 2); //display 2 decimal places
  Serial.println(" C");

  Serial.print("Pressure: ");
  Serial.print(pressure, 0); //whole number only.
  Serial.println(" Pa");

  Serial.print("Standard Atmosphere: ");
  Serial.println(atm, 4); //display 4 decimal places

  Serial.print("Altitude: ");
  Serial.print(altitude, 2); //display 2 decimal places
  Serial.println(" M");

  Serial.println();//line break
*/

//  delay(1000); //wait a second and get values again.
  }
}

/********************************************************************************/
/********************************************************************************/
// BMP085 / BMP180 routines
/********************************************************************************/
// Stores all of the bmp085's calibration values into global variables
// Calibration values are required to calculate temp and pressure
// This function should be called at the beginning of the program
void bmp085Calibration()
{
  Serial.print("Calibrating ... ");
  ac1 = bmp085ReadInt(0xAA);
  ac2 = bmp085ReadInt(0xAC);
  ac3 = bmp085ReadInt(0xAE);
  ac4 = bmp085ReadInt(0xB0);
  ac5 = bmp085ReadInt(0xB2);
  ac6 = bmp085ReadInt(0xB4);
  b1 = bmp085ReadInt(0xB6);
  b2 = bmp085ReadInt(0xB8);
  mb = bmp085ReadInt(0xBA);
  mc = bmp085ReadInt(0xBC);
  md = bmp085ReadInt(0xBE);
  Serial.println("Calibrated");
}

/********************************************************************************/
// Calculate temperature in deg C
float bmp085GetTemperature(unsigned int ut){
  long x1, x2;

  x1 = (((long)ut - (long)ac6)*(long)ac5) >> 15;
  x2 = ((long)mc << 11)/(x1 + md);
  b5 = x1 + x2;

  float temp = ((b5 + 8)>>4);
  temp = temp /10;

  return temp;
}

/********************************************************************************/
// Calculate pressure given up
// calibration values must be known
// b5 is also required so bmp085GetTemperature(...) must be called first.
// Value returned will be pressure in units of Pa.
long bmp085GetPressure(unsigned long up){
  long x1, x2, x3, b3, b6, p;
  unsigned long b4, b7;

  b6 = b5 - 4000;
  // Calculate B3
  x1 = (b2 * (b6 * b6)>>12)>>11;
  x2 = (ac2 * b6)>>11;
  x3 = x1 + x2;
  b3 = (((((long)ac1)*4 + x3)<<OSS) + 2)>>2;

  // Calculate B4
  x1 = (ac3 * b6)>>13;
  x2 = (b1 * ((b6 * b6)>>12))>>16;
  x3 = ((x1 + x2) + 2)>>2;
  b4 = (ac4 * (unsigned long)(x3 + 32768))>>15;

  b7 = ((unsigned long)(up - b3) * (50000>>OSS));
  if (b7 < 0x80000000)
    p = (b7<<1)/b4;
  else
    p = (b7/b4)<<1;

  x1 = (p>>8) * (p>>8);
  x1 = (x1 * 3038)>>16;
  x2 = (-7357 * p)>>16;
  p += (x1 + x2 + 3791)>>4;

  long temp = p;
  return temp;
}

/********************************************************************************/
// Read 1 byte from the BMP085 at 'address'
char bmp085Read(byte address)
{
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();
  Wire.requestFrom(BMP085_ADDRESS, 1);
  while(!Wire.available()) {};
  return Wire.read();
}

/********************************************************************************/
// Read 2 bytes from the BMP085
// First byte will be from 'address'
// Second byte will be from 'address'+1
int bmp085ReadInt(byte address)
{
  unsigned char msb, lsb;

  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();

  Wire.requestFrom(BMP085_ADDRESS, 2);
  while(Wire.available()<2)
    ;
  msb = Wire.read();
  lsb = Wire.read();

  return (int) msb<<8 | lsb;
}

/********************************************************************************/
// Read the uncompensated temperature value
unsigned int bmp085ReadUT(){
  unsigned int ut;

  // Write 0x2E into Register 0xF4
  // This requests a temperature reading
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write((byte)0xF4);
  Wire.write((byte)0x2E);
  Wire.endTransmission();

  // Wait at least 4.5ms
  delay(5);

  // Read two bytes from registers 0xF6 and 0xF7
  ut = bmp085ReadInt(0xF6);
  return ut;
}

/********************************************************************************/
// Read the uncompensated pressure value
unsigned long bmp085ReadUP(){

  unsigned char msb, lsb, xlsb;
  unsigned long up = 0;

  // Write 0x34+(OSS<<6) into register 0xF4
  // Request a pressure reading w/ oversampling setting
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(0xF4);
  Wire.write(0x34 + (OSS<<6));
  Wire.endTransmission();

  // Wait for conversion, delay time dependent on OSS
  delay(2 + (3<<OSS));

  // Read register 0xF6 (MSB), 0xF7 (LSB), and 0xF8 (XLSB)
  msb = bmp085Read(0xF6);
  lsb = bmp085Read(0xF7);
  xlsb = bmp085Read(0xF8);

  up = (((unsigned long) msb << 16) | ((unsigned long) lsb << 8) | (unsigned long) xlsb) >> (8-OSS);

  return up;
}

/********************************************************************************/
void writeRegister(int deviceAddress, byte address, byte val) {
  Wire.beginTransmission(deviceAddress); // start transmission to device 
  Wire.write(address);       // send register address
  Wire.write(val);         // send value to write
  Wire.endTransmission();     // end transmission
}

/********************************************************************************/
int readRegister(int deviceAddress, byte address){

  int v;
  Wire.beginTransmission(deviceAddress);
  Wire.write(address); // register to read
  Wire.endTransmission();

  Wire.requestFrom(deviceAddress, 1); // read a byte

  while(!Wire.available()) {
    // waiting
  }

  v = Wire.read();
  return v;
}

/********************************************************************************/
float calcAltitude(float pressure){

  float A = pressure/101325;
  float B = 1/5.25588;
  float C = pow(A,B);
  C = 1 - C;
  C = C /0.0000225577;

  return C;
}
/********************************************************************************/
/********************************************************************************/

